﻿Imports System.Xml
Imports System.Threading
Imports System.Data.OleDb

Imports MySql.Data.MySqlClient

Imports CrystalDecisions.CrystalReports.Engine
Imports CrystalDecisions.Windows.Forms
Imports CrystalDecisions.Shared

Module Database
    Public Conn As MySqlConnection = SetDbConnectionString()
    Public DbAdapter As New MySqlDataAdapter
    Public DbDataReader As MySqlDataReader
    Public DbCmd As New MySqlCommand
    Public DtResult As DataTable

    ''' <summary>
    ''' Sets the connection string of the public object which is [Conn}.
    ''' </summary>
    ''' <returns>The connection string for MysqlConnection</returns>
    ''' <remarks>Sets a MySqlConnection</remarks>
    Private Function SetDbConnectionString() As MySqlConnection
        Dim xmlDoc As New XmlDocument()
        xmlDoc.Load(Application.StartupPath & "/Helpers/dbconfig.xml")
        Dim nodes As XmlNodeList = xmlDoc.DocumentElement.SelectNodes("/server")

        Dim DbName As String = ""
        Dim DbHostName As String = ""
        Dim DbUsername As String = ""
        Dim DbPassword As String = ""

        For Each node As XmlNode In nodes
            DbHostName = node.SelectSingleNode("host-name").InnerText
            DbName = node.SelectSingleNode("database-name").InnerText
            DbUsername = node.SelectSingleNode("username").InnerText
            DbPassword = node.SelectSingleNode("password").InnerText
        Next

        Dim DbSetup As String = String.Format("server={0}; user id={1}; password={2}; database={3}; pooling=false", DbHostName, DbUsername, DbPassword, DbName)

        Return New MySqlConnection(DbSetup)
    End Function

    ''' <summary>
    ''' Toggles the connection of the database connection. If it is open then close it otherwise open it if it closed.
    ''' </summary>
    ''' <remarks>Toggles the connection of the database connection.</remarks>
    Private Sub SetDbConnection()
        Try
            If Conn.State = ConnectionState.Open Then Conn.Close()
            Conn.Open()
        Catch ex As Exception
            MessageBox.Show(ex.Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    ''' <summary>
    ''' Executes a query to the database.
    ''' </summary>
    ''' <param name="Query">The query that will be exceuted.</param>
    ''' <param name="DtgResult">The datagridview where the resut will take place.</param>
    ''' <remarks>Excecutes a query in the database.</remarks>
    Public Sub Query(ByVal Query As String, ByVal DtgResult As DataGridView)
        Try
            Dim QueryResultCount As Integer = 0
            DtResult = New DataTable

            SetDbConnection()
            With DbCmd
                .Connection = Conn
                .CommandText = Query
            End With

            QueryResultCount = DbCmd.ExecuteNonQuery()
            If QueryResultCount < 0 Then
                MessageBox.Show("Unable to load data!", PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Warning)
            End If

            DbAdapter.SelectCommand = DbCmd
            DbAdapter.Fill(DtResult)

            DtgResult.DataSource = DtResult

            With DtgResult
                .AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill
                .AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.AllCells
                .ColumnHeadersDefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter
                .RowHeadersVisible = False
                .SelectionMode = DataGridViewSelectionMode.FullRowSelect
                For Each Col As DataGridViewColumn In DtgResult.Columns
                    If Col.HeaderCell.Value.Contains("id") Then
                        Col.Visible = False
                    End If
                Next
            End With
        Catch ex As Exception
            MessageBox.Show(ex.Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    ''' <summary>
    ''' Executes a query to the database and initialize a messagebox when the query is executed successfully! 
    ''' </summary>
    ''' <param name="Message">The message that wil be initialize after the query is successfully executed</param>
    ''' <param name="Query">The query that will be executed in the database.</param>
    ''' <remarks>Executes a query and initialize a messagebox.</remarks>
    Public Sub Query(ByVal Message As String, ByVal Query As String)
        Try

            Dim QueryResultCount As Integer = 0
            DtResult = New DataTable

            SetDbConnection()

            If Query IsNot String.Empty Then
                With DbCmd
                    .Connection = Conn
                    .CommandText = Query
                End With

                QueryResultCount = DbCmd.ExecuteNonQuery()
                If QueryResultCount < 0 Then
                    MessageBox.Show("Unable to load data!", PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Warning)
                Else
                    MessageBox.Show(Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Information)
                End If
            End If

        Catch ex As Exception

            MessageBox.Show(ex.Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

        End Try
    End Sub

    ''' <summary>
    ''' Executes a query directly to the database.
    ''' </summary>
    ''' <param name="Query">The query that will be executed in the database.</param>
    ''' <remarks>Executes a query and initialize a messagebox.</remarks>
    Public Sub Query(ByVal Query As String)
        Try

            Dim QueryResultCount As Integer = 0
            DtResult = New DataTable

            SetDbConnection()
            With DbCmd
                .Connection = Conn
                .CommandText = Query
            End With

            QueryResultCount = DbCmd.ExecuteNonQuery()
            If QueryResultCount < 0 Then
                MessageBox.Show("Unable to load data!", PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Warning)
            End If

        Catch ex As Exception

            MessageBox.Show(ex.Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

        End Try
    End Sub


    ''' <summary>
    ''' Query's to the database that returns only a single value.
    ''' </summary>
    ''' <param name="Query">The query that will be executed.</param>
    ''' <returns>The single result form the query.</returns>
    ''' <remarks>Query's to the database that returns only a single value.</remarks>
    Public Function GetSingleValue(ByVal Query As String) As String
        Dim Result As String = New String("")

        Try
            SetDbConnection()
            DbCmd = New MySqlCommand(Query, Conn)

            DbDataReader = DbCmd.ExecuteReader
            While DbDataReader.Read
                If Not IsDBNull(DbDataReader(0)) Then
                    Result = DbDataReader(0)
                End If
            End While
        Catch ex As Exception
            MessageBox.Show(ex.Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try

        Return Result
    End Function

    ''' <summary>
    ''' Sets an autocompletion to a textbox from a database.
    ''' </summary>
    ''' <param name="SqlQuery">The query for getting the specific column's data.</param>
    ''' <param name="TargetTextBox">The text where the autocompletion will put into action.</param>
    ''' <remarks>Adds an autocompletion to a textbox from a database.</remarks>
    Public Sub SetAutoCompletion(ByVal SqlQuery As String, ByVal TargetTextBox As TextBox)
        Dim Result As New AutoCompleteStringCollection

        SetDbConnection()

        Try
            DbCmd = New MySqlCommand(SqlQuery, Conn)

            DbDataReader = DbCmd.ExecuteReader
            While DbDataReader.Read()
                Result.Add(DbDataReader.GetValue(0))
            End While
            DbDataReader.Close()
        Catch ex As Exception
            MessageBox.Show(ex.Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Warning)
        End Try

        TargetTextBox.AutoCompleteMode = AutoCompleteMode.SuggestAppend
        TargetTextBox.AutoCompleteSource = AutoCompleteSource.CustomSource
        TargetTextBox.AutoCompleteCustomSource = Result
    End Sub

    ''' <summary>
    ''' Get all the records of a specific column of a table by using an id.
    ''' </summary>
    ''' <param name="SqlQuery">The query to get all the cols</param>
    ''' <param name="TargetControl">The combobox where the output will go.</param>
    ''' <remarks>Get all the records in a table.</remarks>
    Public Sub SetAutoCompletion(ByVal SqlQuery As String, ByVal TargetControl As ComboBox)
        TargetControl.Items.Clear()

        Try
            SetDbConnection()

            DbCmd = New MySqlCommand(SqlQuery, Conn)
            DbDataReader = DbCmd.ExecuteReader

            While DbDataReader.Read()
                TargetControl.Items.Add(DbDataReader.GetValue(0))
            End While
            TargetControl.AutoCompleteSource = AutoCompleteSource.ListItems
        Catch ex As Exception
            MessageBox.Show(ex.Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Warning)
        End Try

        DbDataReader.Close()
    End Sub

    ''' <summary>
    ''' Displays a report to a report viewer.
    ''' </summary>
    ''' <param name="Query">Teh query that will be executed in the report</param>
    ''' <param name="FileName">The filename for the template of the report to be used.</param>
    ''' <param name="ReportViewer">The report viewer where all the data will take place.</param>
    ''' <remarks>Displays a report to a report viewer.</remarks>
    Public Sub PrintReport(ByVal Query As String, ByVal FileName As String, ByVal ReportViewer As CrystalReportViewer)

        Dim Report As New ReportDocument
        DtResult = New DataTable

        ReportViewer.ReportSource = Nothing

        SetDbConnection()
        Try
            SetDbConnection()
            With DbCmd
                .Connection = Conn
                .CommandText = Query
            End With

            DbAdapter.SelectCommand = DbCmd
            DbAdapter.Fill(DtResult)
        Catch ex As Exception

            MessageBox.Show(ex.Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try

        Report.Load(Application.StartupPath & "\Reports\" & FileName & ".rpt")
        Report.SetDataSource(DtResult)

        ReportViewer.ReportSource = Report
        ReportViewer.Refresh()
    End Sub
End Module
